(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs/Subject'), require('rxjs/add/operator/takeUntil'), require('rxjs/BehaviorSubject'), require('rxjs/add/operator/filter')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', 'rxjs/Subject', 'rxjs/add/operator/takeUntil', 'rxjs/BehaviorSubject', 'rxjs/add/operator/filter'], factory) :
	(factory((global['angular-contents'] = {}),global.ng.core,global.Rx,global.Rx.Observable.prototype,global.Rx));
}(this, (function (exports,core,Subject,takeUntil,BehaviorSubject) { 'use strict';

var ContentsDirective = (function () {
    function ContentsDirective() {
        var _this = this;
        this._onScroll$ = new Subject.Subject();
        this._activeSection$ = new BehaviorSubject.BehaviorSubject(null);
        this.scrollFun = function (event) { return _this._onScroll$.next(event); };
    }
    /**
     * @return {?}
     */
    ContentsDirective.prototype.ngOnInit = function () {
        this.unsubscribeScrollEventListener();
        this.subscribeScrollEventListener();
    };
    /**
     * @return {?}
     */
    ContentsDirective.prototype.ngOnChanges = function () {
        this.unsubscribeScrollEventListener();
        this.subscribeScrollEventListener();
    };
    /**
     * @return {?}
     */
    ContentsDirective.prototype.ngOnDestroy = function () {
        this.unsubscribeScrollEventListener();
    };
    /**
     * @return {?}
     */
    ContentsDirective.prototype.subscribeScrollEventListener = function () {
        (this.scrollingView || document).addEventListener('scroll', this.scrollFun, false);
    };
    /**
     * @return {?}
     */
    ContentsDirective.prototype.unsubscribeScrollEventListener = function () {
        (this.scrollingView || document).removeEventListener('scroll', this.scrollFun, false);
    };
    /**
     * @return {?}
     */
    ContentsDirective.prototype.activeSection = function () {
        return this._activeSection$
            .asObservable()
            .filter(function (section) { return !!section; }); // Prevent returning null values.
    };
    return ContentsDirective;
}());
ContentsDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[contents]',
                exportAs: 'contents',
            },] },
];
/**
 * @nocollapse
 */
ContentsDirective.ctorParameters = function () { return []; };
ContentsDirective.propDecorators = {
    'scrollingView': [{ type: core.Input },],
};
var ContentsLinkDirective = (function () {
    /**
     * @param {?} contents
     */
    function ContentsLinkDirective(contents) {
        this.contents = contents;
        this.ngUnsubscribe = new Subject.Subject();
        this.active = false;
    }
    /**
     * @return {?}
     */
    ContentsLinkDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.contents._activeSection$
            .takeUntil(this.ngUnsubscribe)
            .subscribe(function (sectionName) {
            _this.active = "#" + sectionName === _this.href;
        });
    };
    /**
     * @return {?}
     */
    ContentsLinkDirective.prototype.ngOnDestroy = function () {
        this.ngUnsubscribe.next();
        this.ngUnsubscribe.complete();
    };
    return ContentsLinkDirective;
}());
ContentsLinkDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[contentsLink]',
                exportAs: 'contentsLink',
            },] },
];
/**
 * @nocollapse
 */
ContentsLinkDirective.ctorParameters = function () { return [
    { type: ContentsDirective, decorators: [{ type: core.Host },] },
]; };
ContentsLinkDirective.propDecorators = {
    'href': [{ type: core.Input },],
    'active': [{ type: core.HostBinding, args: ['class.active',] },],
};
/**
 * @param {?} element
 * @return {?}
 */
function getAbsoluteHeight(element) {
    var /** @type {?} */ styles = window.getComputedStyle(element);
    var /** @type {?} */ margin = parseFloat(styles.marginTop || '0') +
        parseFloat(styles.marginBottom || '0');
    return Math.ceil(element.offsetHeight + margin);
}
/**
 * @return {?}
 */
function documentOffset() {
    return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}
var ContentsSectionDirective = (function () {
    /**
     * @param {?} contents
     * @param {?} elementRef
     */
    function ContentsSectionDirective(contents, elementRef) {
        this.contents = contents;
        this.elementRef = elementRef;
        this.ngUnsubscribe = new Subject.Subject();
    }
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.detectActiveChanges();
        this.contents._onScroll$
            .takeUntil(this.ngUnsubscribe)
            .subscribe(function (event) {
            _this.detectActiveChanges();
        });
    };
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.ngOnDestroy = function () {
        this.ngUnsubscribe.next();
        this.ngUnsubscribe.complete();
    };
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.detectActiveChanges = function () {
        if (this.isInRange() || !this.contents._activeSection$.value) {
            this.contents._activeSection$.next(this.contentsSection);
        }
    };
    /**
     * @return {?}
     */
    ContentsSectionDirective.prototype.isInRange = function () {
        var /** @type {?} */ pageOffset = this.contents.scrollingView ? this.contents.scrollingView.scrollTop : documentOffset();
        var /** @type {?} */ element = this.elementRef.nativeElement;
        var /** @type {?} */ offset = element.offsetTop;
        var /** @type {?} */ height = getAbsoluteHeight(element);
        return pageOffset >= offset && pageOffset <= offset + height;
    };
    return ContentsSectionDirective;
}());
ContentsSectionDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[contentsSection]',
                exportAs: 'contentsSection',
            },] },
];
/**
 * @nocollapse
 */
ContentsSectionDirective.ctorParameters = function () { return [
    { type: ContentsDirective, decorators: [{ type: core.Host },] },
    { type: core.ElementRef, },
]; };
ContentsSectionDirective.propDecorators = {
    'contentsSection': [{ type: core.HostBinding, args: ['id',] }, { type: core.Input },],
};
var ContentsTableDirective = (function () {
    /**
     * @param {?} elementRef
     */
    function ContentsTableDirective(elementRef) {
        this.elementRef = elementRef;
        this.sticky = false;
        this.marginTop = '0px';
    }
    /**
     * @return {?}
     */
    ContentsTableDirective.prototype.ngOnInit = function () {
        this.updateStickiness();
    };
    /**
     * Check whether the Table of Contents should be a sticky, to keep itself visible while the user
     * scrolls.
     * @return {?}
     */
    ContentsTableDirective.prototype.updateStickiness = function () {
        var /** @type {?} */ pageOffset = window.pageYOffset;
        var /** @type {?} */ parentElement = this.elementRef.nativeElement.parentNode;
        var /** @type {?} */ parentOffset = parentElement.offsetTop;
        var /** @type {?} */ parentHeight = getAbsoluteHeight(parentElement);
        var /** @type {?} */ element = this.elementRef.nativeElement;
        var /** @type {?} */ elementInnerHeight = element.offsetHeight;
        if (pageOffset + elementInnerHeight > parentOffset + parentHeight) {
            // Use a fixed margin-top instead when scrolling past the parent container.
            this.sticky = false;
            this.marginTop = parentHeight - elementInnerHeight + "px";
        }
        else {
            // Use CSS sticky when scrolling into the parent container.
            this.marginTop = '0px';
            this.sticky = pageOffset > parentOffset;
        }
    };
    /**
     * @param {?} target
     * @return {?}
     */
    ContentsTableDirective.prototype.getHeight = function (target) {
        var /** @type {?} */ element = this.elementRef.nativeElement;
        var /** @type {?} */ styles = window.getComputedStyle(element);
        var /** @type {?} */ margin = parseFloat(styles.marginTop || '0') +
            parseFloat(styles.marginBottom || '0');
        return Math.ceil(element.offsetHeight + margin);
    };
    return ContentsTableDirective;
}());
ContentsTableDirective.decorators = [
    { type: core.Directive, args: [{
                selector: '[contentsTable]',
                exportAs: 'contentsTable',
            },] },
];
/**
 * @nocollapse
 */
ContentsTableDirective.ctorParameters = function () { return [
    { type: core.ElementRef, },
]; };
ContentsTableDirective.propDecorators = {
    'sticky': [{ type: core.HostBinding, args: ['class.sticky',] },],
    'marginTop': [{ type: core.HostBinding, args: ['style.margin-top',] },],
    'updateStickiness': [{ type: core.HostListener, args: ['window:scroll', ['$event'],] },],
};
var ContentsModule = (function () {
    function ContentsModule() {
    }
    return ContentsModule;
}());
ContentsModule.decorators = [
    { type: core.NgModule, args: [{
                imports: [],
                exports: [
                    ContentsLinkDirective,
                    ContentsSectionDirective,
                    ContentsTableDirective,
                    ContentsDirective,
                ],
                declarations: [
                    ContentsLinkDirective,
                    ContentsSectionDirective,
                    ContentsTableDirective,
                    ContentsDirective,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ContentsModule.ctorParameters = function () { return []; };

exports.ContentsLinkDirective = ContentsLinkDirective;
exports.ContentsSectionDirective = ContentsSectionDirective;
exports.ContentsTableDirective = ContentsTableDirective;
exports.ContentsDirective = ContentsDirective;
exports.ContentsModule = ContentsModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=angular-contents.umd.js.map
